@@ -1,11 +1,12 @@
 import inspect
-from typing import Any 
+from typing import Any, Optional, Type 
 
-def get_members(obj: Any, no_parent: bool = True):
+def get_members(obj: Any, no_parent: bool = True, python_inherit: Optional[Type] = None):
     """this function return member functions that keep def order.
     """
     this_cls = type(obj)
     if not no_parent:
+        assert python_inherit is None, "not implemented"
         res = inspect.getmembers(this_cls, inspect.isfunction)
         # inspect.getsourcelines need to read file, so .__code__.co_firstlineno
         # is greatly faster than it.
@@ -15,6 +16,10 @@ def get_members(obj: Any, no_parent: bool = True):
     parents = inspect.getmro(this_cls)[1:]
     parents_methods = set()
     for parent in parents:
+        if python_inherit is not None:
+            if parent is python_inherit:
+                # add inherited python members
+                continue
         members = inspect.getmembers(parent, predicate=inspect.isfunction)
         parents_methods.update(members)
 
