@@ -1,5 +1,6 @@
 import ast
 from collections import OrderedDict
+import dataclasses
 from enum import Enum
 from typing import Callable, Dict, List, Optional, Set, Tuple, Union
 
@@ -117,8 +118,15 @@ def _anno_parser(node: ast.AST, imports: List[str],
         msg += "but we get ast node {}".format(ast.dump(node))
         raise ValueError(msg)
 
-
+@dataclasses.dataclass
 class TemplateTypeStmt(object):
+    name: str
+    args: List["TemplateTypeStmt"]
+    not_template: bool
+    invalid: bool = False
+    exist_anno: str = ""
+    is_ptr: bool = False
+    
     NameToHandler = {
         "int":
         lambda args: "int",
@@ -198,20 +206,6 @@ class TemplateTypeStmt(object):
         lambda args: args[0].to_pyanno(),
     }  # type: Dict[str, Callable[[List["TemplateTypeStmt"]], str]]
 
-    def __init__(self,
-                 name: str,
-                 args: List["TemplateTypeStmt"],
-                 not_template: bool,
-                 invalid: bool = False,
-                 exist_anno: str = "",
-                 is_ptr: bool = False):
-        self.name = name
-        self.args = args
-        self.not_template = not_template
-        self.invalid = invalid
-        self.exist_anno = exist_anno
-        self.is_ptr = is_ptr
-
     def to_pyanno(self) -> str:
         if self.invalid:
             return "Any"
@@ -232,8 +226,18 @@ def _simple_template_type_parser_recursive(
         stmt: str, begin: int, end: int, bracket_pair: Dict[int, int],
         exist_annos: Dict[str, str]) -> TemplateTypeStmt:
     # remove const
-    if stmt[:5] == "const" and stmt[5] == " ":
+    if stmt[begin:begin + 5] == "const" and stmt[begin + 5] == " ":
         begin += 5
+    # remove constant, thread, threadgroup, device in metal
+    if stmt[begin:begin + 8] == "constant" and stmt[begin + 8] == " ":
+        begin += 8
+    if stmt[begin:begin + 6] == "thread" and stmt[begin + 6] == " ":
+        begin += 6
+    if stmt[begin:begin + 11] == "threadgroup" and stmt[begin + 11] == " ":
+        begin += 11
+    if stmt[begin:begin + 6] == "device" and stmt[begin + 6] == " ":
+        begin += 6
+
     # remove tailing const, pointer and ref
     is_ptr = False 
     if stmt.endswith("__restrict__"):
@@ -247,7 +251,6 @@ def _simple_template_type_parser_recursive(
         end -= 1
     elif stmt[-1] == "&":
         end -= 1
-
     stmt = stmt[:end]
     stmt2 = stmt.strip()
     end -= len(stmt) - len(stmt2)
@@ -922,15 +925,19 @@ def _generate_python_interface_class(cls_name: str,
     # Class EnumName:
     for ec in enum_classes:
         ec_items = []  # type: List[Union[Block, str]]
+        ec_enum_items = []  # type: List[Union[Block, str]]
+
         enum_type = "EnumValue"
         if ec.scoped:
             enum_type = "EnumClassValue"
 
-        prefix = "class {}:".format(ec.name)
+        prefix = "class {}(enum.Enum):".format(ec.name)
         for key, value in ec.items:
-            ec_items.append("{k} = {ectype}({v}) # type: {ectype}".format(
+            ec_items.append("{k} = {clsname}.{k}".format(
+                k=key, v=value, clsname=ec.name))
+            ec_enum_items.append("{k} = {v}".format(
                 k=key, v=value, ectype=enum_type))
-        def_items = ec_items.copy()
+        def_items = ec_enum_items.copy()
         def_items.append("@staticmethod")
         def_items.append(
             "def __members__() -> Dict[str, {}]: ...".format(enum_type))
@@ -1112,7 +1119,7 @@ class Pybind11SplitMain(ParameterizedClass):
         TODO insert docstring if exists
         """
         init_import = "from typing import overload, Any, Callable, Dict, List, Optional, Set, Tuple, Type, Union"
-        init_pccm_import = "from pccm.stubs import EnumValue, EnumClassValue"
+        init_pccm_import = "from pccm.stubs import EnumValue, EnumClassValue, enum"
         ns_to_interfaces = OrderedDict()  # type: Dict[str, List[Block]]
         ns_to_imports = OrderedDict()  # type: Dict[str, List[str]]
         ns_to_interface = OrderedDict()  # type: Dict[str, str]
